<?php

function voting_actions_voting_actions_conditions() {
  $condition = array();

  $condition['individual_vote'] = array(
    'name' => t('Individual vote'),
    'description' => t('Return true if any item in the collection of cast votes matches these criteria.'),
    'eval_handler' => 'voting_actions_individual_vote_handler',
    'edit_handler' => 'voting_actions_vote_form',
  );

  $condition['all_votes'] = array(
    'name' => t('All individual votes'),
    'description' => t('Return true if all items in the collection of cast votes match these criteria.'),
    'eval_handler' => 'voting_actions_all_votes_handler',
    'edit_handler' => 'voting_actions_vote_form',
  );

  $condition['vote_result'] = array(
    'name' => t('Vote result'),
    'description' => t('Return true if a vote result matches these criteria.'),
    'eval_handler' => 'voting_actions_vote_result_handler',
    'edit_handler' => 'voting_actions_vote_result_form',
  );

  $condition['content_property'] = array(
    'name' => t('Content property'),
    'description' => t('Return true if the selected property of the content being voted on matches these criteria.'),
    'eval_handler' => 'voting_actions_content_property_handler',
    'edit_handler' => 'voting_actions_content_property_form',
  );

  $condition['content_author_perm'] = array(
    'name' => t('Content author has permission'),
    'description' => t('Return true if the author of the content being voted on matches a certain permission.'),
    'eval_handler' => 'voting_actions_content_author_perm_handler',
    'edit_handler' => 'voting_actions_content_author_perm_form',
  );

  return $condition;
}

function _voting_actions_string_operators() {
  return array(
    '==' => t('Is'),
    'contains' => t('Contains'),
    'excludes' => t('Does not contain'),
    'starts' => t('Stars with'),
    'ends' => t('Ends with'),
  );
}

function _voting_actions_number_operators() {
  return array(
    '<' => t('Is less than'),
    '<=' => t('Is less than or equal to'),
    '==' => t('Is equal to'),
    '>=' => t('Is greater than or equal to'),
    '>' => t('Is greater than'),
    '!=' => t('Is not equal to'),
  );
}

function _voting_actions_boolean_operators() {
  return array(
    '==' => t('Is'),
    '!=' => t('Is not'),
  );
}



/**
 * Processing/evaluation handlers for custom voting action conditions.
 */

// Returns true if ONE vote matches the criteria specified.
function voting_actions_individual_vote_handler($context, $condition) {
  foreach ($context['votes'] as $vote) {
    $result = TRUE;
    if (!empty($condition['data']['value_type'])) {
      if ($condition['data']['value_type'] != $vote->value_type) {
        continue;
      }
    }
    if (!empty($condition['data']['tag'])) {
      if ($condition['data']['tag'] != $vote->tag) {
        continue;
      }
    }
    if (_voting_actions_compare($vote->value, $condition['data']['operator'], $condition['data']['value'])) {
      return TRUE;
    }
  }
  return FALSE;
}

function voting_actions_all_votes_handler($context, $condition) {
  foreach ($context['votes'] as $vote) {
    if (!empty($condition['data']['value_type'])) {
      if ($condition['data']['value_type'] != $vote->value_type) {
        continue;
      }
    }
    if (!empty($condition['data']['tag'])) {
      if ($condition['data']['tag'] != $vote->tag) {
        continue;
      }
    }
    if (_voting_actions_compare($vote->value, $condition['data']['operator'], $condition['data']['value']) == FALSE) {
      return FALSE;
    }
  }
  return TRUE;
}

function voting_actions_vote_form($values = array()) {
  $values += array(
    'value_type' => '',
    'tag' => '',
    'operator' => '',
    'value' => '',
  );

  $form['value_type'] = array(
    '#type' => 'select',
    '#options' => _voting_actions_table_values('value_type', 'vote', array('' => t('(Ignore)'), 'percent' => t('Percent'), 'points' => t('Points'))),
    '#default_value' => $values['value_type'],
  );

  $form['tag'] = array(
    '#type' => 'select',
    '#options' => _voting_actions_table_values('tag', 'vote', array('' => t('(Ignore)'), 'percent' => t('Vote'))),
    '#default_value' => $values['tag'],
  );

  $form['operator'] = array(
    '#type' => 'select',
    '#options' => _voting_actions_number_operators(),
    '#default_value' => $values['operator'],
  );
  
  $form['value'] = array(
    '#type' => 'textfield',
    '#default_value' => $values['value'],
  );
  return $form;
}



// Returns true if all vote results match the criteria.
function voting_actions_vote_result_handler($context, $condition) {
  foreach ($context['results'] as $vote) {
    if (!empty($condition['data']['value_type'])) {
      if ($condition['data']['value_type'] != $vote->value_type) {
        continue;
      }
    }
    if (!empty($condition['data']['tag'])) {
      if ($condition['data']['tag'] != $vote->tag) {
        continue;
      }
    }
    if (!empty($condition['data']['function'])) {
      if ($condition['data']['function'] != $vote->function) {
        continue;
      }
    }
    if (_voting_actions_compare($vote->value, $condition['data']['operator'], $condition['data']['value']) == FALSE) {
      return FALSE;
    }
  }
  return TRUE;
}

function voting_actions_vote_results_form($values = array()) {
  $values += array(
    'value_type' => '',
    'tag' => '',
    'function' => '',
    'operator' => '',
    'value' => '',
  );

  $form['value_type'] = array(
    '#type' => 'select',
    '#options' => _voting_actions_table_values('value_type', 'cache', array('' => t('(Ignore)'), 'percent' => t('Percent'), 'points' => t('Points'))),
    '#default_value' => $values['value_type'],
  );

  $form['tag'] = array(
    '#type' => 'select',
    '#options' => _voting_actions_table_values('tag', 'cache', array('' => t('(Ignore)'), 'percent' => t('Vote'))),
    '#default_value' => $values['tag'],
  );

  $form['function'] = array(
    '#type' => 'select',
    '#options' => _voting_actions_table_values('function', 'cache', array('' => t('(Ignore)'), 'average' => t('Average'), 'count' => t('Count'))),
    '#default_value' => $values['function'],
  );

  $form['operator'] = array(
    '#type' => 'select',
    '#options' => _voting_actions_number_operators(),
    '#default_value' => $values['operator'],
  );
  
  $form['value'] = array(
    '#type' => 'textfield',
    '#default_value' => $values['value'],
  );
  return $form;
}


// Returns true if a specific property of the content object matches the criteria.
function voting_actions_content_property_handler($context, $condition) {
  if (!empty($condition['data']['property']) && !empty($context['contet']->$condition['data']['property'])) {
    return _voting_actions_compare($context['content']->$condition['data']['property'], $condition['data']['operator'], $condition['data']['value']);
  }
  return FALSE;
}

// Returns true if the author's content has the specified permission.
function voting_actions_content_author_perm_handler($context, $condition) {
  if (!empty($context['content']->uid)) {
    $user = user_load(array('uid' => $context['content']->uid));
    return _voting_actions_compare(user_access($user, $condition['data']['perm']), $condition['data']['operator'], TRUE);
  }
  return FALSE;
}

function _voting_actions_table_values($field = 'tag', $table = 'vote', $force = array()) {
  static $cached;
  if (!isset($cached[$table][$field])) {
    $results = db_query("SELECT DISTINCT %s FROM {votingapi_%s} WHERE content_type = 'node'", $field, $table);
    while ($result = db_fetch_object($results)) {
      $cached[$table][$field][$result->$field] = ucfirst($result->$field);
    }
  }
  return empty($cached[$table][$field]) ? $force : $force += $cached[$table][$field];
}

// Internal helper fuction.
function _voting_actions_compare($left, $op, $right, $default = TRUE) {
  if (empty($left) || empty($default)) {
    return $default;
  }

  switch ($op) {
    case "==":
      return ($left == $right);
      break;
    case "!=":
      return ($left != $right);
      break;
    case "<":
      return ($left < $right);
      break;
    case "<=":
      return ($left <= $right);
      break;
    case ">=":
      return ($left >= $right);
      break;
    case ">":
      return ($left > $right);
      break;
    case "contains":
      if (is_array($left)) {
        $tmp = array_flip($left);
        return (isset($tmp[$right]));
      }
      else {
      }
      break;
    case "excludes":
      break;
    case "starts":
      break;
    case "ends":
      break;
  }
  return $default;
}