<?php

// Contains UI management functions for the voting_actions module.

function voting_actions_admin_page() {
  voting_actions_get_sets(TRUE);
  $db_sets = _voting_actions_load_sets_from_db();
  foreach ($db_sets as $name => $set) {
    $vasid = $set['vasid'];
    $links = array();
    $links['voting_actions_edit'] = array(
       'title' => theme('image', drupal_get_path('module', 'voting_actions') . '/icons/gtk-edit.png'),
       'html' => TRUE,
       'href' => "admin/settings/voting_actions/edit/$vasid",
       'attributes' => array('title' => t('Edit this set')),
    );
    $links['voting_actions_export'] = array(
       'title' => theme('image', drupal_get_path('module', 'voting_actions') . '/icons/document-save.png'),
       'html' => TRUE,
       'href' => "admin/settings/voting_actions/export/$name/db",
       'attributes' => array('title' => t('Export this set')),
    );
    $links['voting_actions_delete'] = array(
       'title' => theme('image', drupal_get_path('module', 'voting_actions') . '/icons/edit-delete.png'),
       'html' => TRUE,
       'href' => "admin/settings/voting_actions/delete/$name",
       'attributes' => array('title' => t('Delete this set')),
    );

    $items[] = array($name, theme('links', $links));
  }

  if (empty($items)) {
    $items[] = array('data' => array(array('data' => '<p class="description">' .t("No action sets have currently been defined. !click to create one.", array('!click' => l(t('Click here'), 'admin/settings/voting_actions/add'))) .'</p>', 'colspan' => 2)));
  }
  $output = theme('table', array(t('Name'), t('Actions')), $items, array("cellpadding" => "3"), t('User-Defined Action Sets'));

  $items = array();
  $module_sets = module_invoke_all('voting_actions_sets');

  foreach ($module_sets as $name => $set) {
    $set += array(
      'source' => 'user',
      'required' => TRUE,
      'condition_mask' => 'AND',
      'weight' => 0,
      'conditions' => array(),
      'actions' => array(),
      'subsets' => array(),
    );
    $set['status'] = voting_actions_get_status($name);

    $links = array();
    if (array_key_exists($name, $db_sets) && $set['status'] != VOTING_ACTIONS_STATE_OVERRIDDEN) {
      voting_actions_set_status($name, VOTING_ACTIONS_STATE_OVERRIDDEN);
      $set['status'] = voting_actions_get_status($name);
    }
    else {
      $links['voting_actions_add'] = array(
         'title' => theme('image', drupal_get_path('module', 'voting_actions') . '/icons/edit-paste.png'),
         'html' => TRUE,
         'href' => "admin/settings/voting_actions/add/$name",
         'attributes' => array('title' => t('Clone this set')),
      );
      $links['voting_actions_export'] = array(
         'title' => theme('image', drupal_get_path('module', 'voting_actions') . '/icons/document-save.png'),
         'html' => TRUE,
         'href' => "admin/settings/voting_actions/export/$name/mod",
         'attributes' => array('title' => t('Export this set')),
      );
    }

    switch($set['status']) {
      case VOTING_ACTIONS_STATE_ENABLED:
        $toggle['voting_actions_disable'] = array(
          'title' => theme('image', drupal_get_path('module', 'voting_actions') . '/icons/media-playback-start.png') . t('Enabled'),
          'html' => TRUE,
          'href' => "admin/settings/voting_actions/disable/$name",
          'attributes' => array('title' => t('Disable this set')),
        );
        break;
      case VOTING_ACTIONS_STATE_DISABLED:
        $toggle['voting_actions_enable'] = array(
          'title' => theme('image', drupal_get_path('module', 'voting_actions') . '/icons/process-stop.png') . t('Disabled'),
          'html' => TRUE,
          'href' => "admin/settings/voting_actions/enable/$name",
          'attributes' => array('title' => t('Enable this set')),
        );
        break;
      default:
        $toggle['voting_actions_enable'] = array(
          'title' => t('Overridden'),
          'attributes' => array('title' => t('Overridden')),
        );
        break;
    }

    $items[] = array($name, theme('links', $toggle), theme('links', $links));
  }

  if (empty($items)) {
    $items[] = array('data' => array(array('data' => '<p class="description">'. t('No default action sets have been exposed.') .'</p>', 'colspan' => '4')));
  }
  $output .= theme('table', array(t('Name'), t('Status'), t('Actions')), $items, array("cellpadding" => "3"), t('Default Action Sets'));

  return $output;
}

function voting_actions_admin_import_page($set_name = NULL) {
  $form['set'] = array(
    '#type' => 'textarea',
    '#title' => t('Import action set'),
    '#rows' => 20
  );
  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t("Submit"),
  );

  return $form;
}
function voting_actions_admin_import_page_validate($form_id, &$form) {
  ob_start();
  eval($form['set']);
  ob_end_clean();

  if (is_array($sets)) {
    foreach($sets as $name => $set) {
      if ($name) {
        $errors = _voting_actions_validate_set($set);
        foreach($errors as $error) {
          form_set_error('', $error);
        }
      }
    }
  }
}

function voting_actions_admin_import_page_submit($formid, $form) {
  ob_start();
  eval($form['set']);
  ob_end_clean();

  if (is_array($sets)) {
    foreach($sets as $name => $set) {
      if ($name) {
        $errors = _voting_actions_validate_set($set);
        if (count($errors) == 0) {
          _voting_actions_insert_set($set);
          voting_actions_get_sets(TRUE);
          drupal_goto('admin/settings/voting_actions');
        }
        else {
          foreach ($errors as $error) {
            drupal_set_message($error);
          }
        }
      }
      else {
        drupal_set_message(t('Unable to get an action set out of that.'));
        return;
      }
    }
  }
}


function voting_actions_admin_export_page($set_name) {
  $code = _voting_actions_create_set_code($set_name);
  $lines = substr_count($code, "\n");
  $form['code'] = array(
    '#type' => 'textarea',
    '#title' => $set['name'],
    '#default_value' => $code,
    '#rows' => max($lines, 20)
  );
  return $form;
}

function _voting_actions_create_set_code($set_name) {
  $data = cache_get('voting_actions_cache');
  $action_sets = unserialize($data->data);
  $set = $action_sets[$set_name];
  if (!empty($set)) {
    unset($set['name']);
    $code = "\n\$sets = array(\n";
    $code .= "'$set_name' => ";
    ob_start();
    var_export($set);
    $code .= ob_get_clean();
    ob_end_clean();
    $code .= "\n);\n";
  }
  
  return $code;
}


function voting_actions_admin_delete_page($set_name) {
  $form['set'] = array('#type' => 'value', '#value' => $set_name);
  return confirm_form(
    $form, 
    t('Are you sure you want to delete %title?', array('%title' => $set_name)), 
    $_GET['destination'] ? $_GET['destination'] : 'admin/settings/voting_actions', 
    t('This action cannot be undone.'),
    t('Delete'), t('Cancel')  
  );
}
function voting_actions_admin_delete_page_submit($formid, $form) {
  $name = $form['set'];
  $sets = _voting_actions_load_sets_from_db();
  if ($sets[$name]) {
    _voting_actions_delete_set($name, $set[$name]);
  }
  return 'admin/settings/voting_actions';
}



function voting_actions_admin_enable_page($set_name) {
  voting_actions_set_status($set_name, VOTING_ACTIONS_STATE_ENABLED);
  drupal_goto('admin/settings/voting_actions');
}

function voting_actions_admin_disable_page($set_name) {
  voting_actions_set_status($set_name, VOTING_ACTIONS_STATE_DISABLED);
  drupal_goto('admin/settings/voting_actions');
}


function voting_actions_admin_add_page($name = NULL) {
  $set = _voting_actions_populate_set_defaults();
  return drupal_get_form('voting_actions_set_form', $set);
}

function voting_actions_admin_edit_page($name = NULL, $db_or_module = 'db') {
  if ($db_or_module == 'db') {
    $sets = _voting_actions_load_sets_from_db();
  }
  else {
    $sets = module_invoke_all('voting_actions_sets');
  }

  if (!empty($name) && !empty($sets[$name])) {
    $set = $sets[$name];
  }
  else {
    $set = array();
  }
  
  $set += _voting_actions_populate_set_defaults();
  return drupal_get_form('voting_actions_set_form', $set);
}


// Generate a one-off form for a specific voting action set.
function voting_actions_set_form($set, $form_values = NULL) {
  if (isset($form_values)) {
    // Build an in-process set before going any further.
    $set = array_merge($form_values, $set);
  
    $set['conditions'] = $set['conditions']['existing'];
    $set['actions'] = $set['actions']['existing'];
    
    unset($set['op']);
    unset($set['submit']);
  }

  $form = _voting_actions_set_form($set);

  $form['conditions'] =  _voting_actions_set_conditions_form($set);

  $form['actions'] =  _voting_actions_set_actions_form($set);

  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Submit'),
  );

  return $form;
}

function _voting_actions_set_form($set) {
  $form = array();

  $form['#multistep'] = TRUE;
  $form['#redirect'] = FALSE;
  $form['#tree'] = TRUE;

  $form['vasid'] = array(
    '#type' => 'hidden',
    '#value' => $set['vasid'],
   );

  $form['name'] = array(
    '#type' => 'textfield',
    '#title' => t('Set name'),
    '#description' => t('A unique name for this set. Stick to lowercase letters, and use underscores instead of spaces.'),
    '#default_value' => $set['name'],
  );
  
  $form['description'] = array(
    '#type' => 'textfield',
    '#title' => t('Description'),
    '#description' => t("A description of this action set's operations. Useful for documenting the intended purpose of the set."),
    '#default_value' => $set['description'],
  );
  
  if (isset($set['parent'])) {
    $form['parent'] = array(
      '#type' => 'hidden',
      '#value' => $set['parent'],
    );
    $form['required'] = array(
      '#type' => 'checkbox',
      '#title' => t('This set is required.'),
      '#return_value' => 1,
      '#default_value' => $set['required'],
      '#description' => t("If this box is checked, this set's conditions must be passed before the parent set's actions can execute."),
    );
  }
  else {
    $form['required'] = array(
      '#type' => 'hidden',
      '#value' => TRUE,
    );
  }

  $form['weight'] = array(
    '#type' => 'hidden',
    '#value' => 0,
  );

  $form['condition_mask'] = array(
    '#type' => 'radios',
    '#title' => t("To execute this set's actions..."),
    '#options' => array('AND' => t('All conditions must be true'), 'OR' => t('At least one condition must be true')),
    '#default_value' => isset($set['condition_mask']) ? $set['condition_mask'] : 'AND',
  );

  return $form;
}
function _voting_actions_set_conditions_form($set) {
  $form = array(
    '#type' => 'fieldset',
    '#title' => t('Conditions'),
    '#collapsible' => TRUE,
    '#collapsed' => FALSE,
    '#description' => t('These conditions determine whether the actions in this set are executed.'),
  );

  $options = array();
  foreach (voting_actions_get_conditions() as $key => $data) {
    $options[$key] = $data['name'];
  }

  $form['existing']['#weight'] = 0;
  foreach($set['conditions'] as $condition) {
    $form['conditions']['existing'][] = voting_actions_condition_form($condition);
  }

  $form['add'] = array(
    '#type' => 'item',
    '#weight' => 100,
    '#title' => t('Add a condition'),
  );
  $form['add']['condition_type'] = array(
    '#type' => 'select',
    '#options' => $options,
  );
  $form['add']['add_condition'] = array(
    '#type' => 'button',
    '#value' => t('Add condition'),
  );
  return $form;
}

function _voting_actions_set_actions_form($set) {
  $form = array(
    '#type' => 'fieldset',
    '#title' => t('Actions'),
    '#collapsible' => TRUE,
    '#collapsed' => FALSE,
    '#description' => t('If the conditions above pass, this list of actions will execute.'),
  );

  $form['existing']['#weight'] = 0;
  foreach($set['actions'] as $action) {
    $form['existing'][] = voting_actions_action_form($action);
  }

  $form['add'] = array(
    '#type' => 'item',
    '#weight' => 100,
    '#title' => t('Add an action'),
  );
  $form['add']['action_type'] = array(
    '#type' => 'select',
    '#options' => _voting_actions_list(),
  );
  $form['add']['add_action'] = array(
    '#type' => 'button',
    '#value' => t('Add action'),
  );
  return $form;
}

function voting_actions_condition_form($condition) {
  $form = array();
  $conditions = voting_actions_get_conditions();
  if ($condition_info = $conditions[$condition['name']]) {
    $form = array(
      '#type' => 'item',
      '#title' => $condition_info['name'],
      '#description' => $condition_info['description'],
      '#attributes' => array('class' => 'container-inline'),
      '#weight' => $condition['weight'],
    );

    if (!empty($condition['vacid'])) {
      $form['vacid'] = array(
        '#type' => 'hidden',
        '#value' => $condition['vacid'],
      );
    }

    $form['name'] = array(
        '#type' => 'hidden',
        '#value' => $condition['name'],
      );

    $form['weight'] = array(
      '#type' => 'hidden',
      '#value' => $condition['weight'],
    );

    if ($function = $condition_info['form_handler'] && function_exists($function)) {
      $form['data'] = $function($condition['data']);
    }
  }
  return $form;
}

function voting_actions_action_form($action) {
  $form = array();
  $actions = actions_actions_map(actions_get_all_actions());

  $form['value'] = array(
    '#type' => 'hidden',
    '#value' => $action,
  );
  $form['description'] = array(
    '#value' => $actions[$action]['description'],
  );
  $form['remove'] = array(
    '#type' => 'markup',
    '#value' => '<br />',
  );
  return $form;
}

function _voting_actions_list() {
  $options = array();
  foreach (actions_actions_map(actions_get_all_actions()) as $aid => $action) {
    $options[$action['type']][$aid] = $action['description'];
  }
  return $options;
}

/**
 * Custom form element to do our nice images.
 */
function voting_actions_elements() {
  $type['imagebutton'] = array('#input' => TRUE, '#button_type' => 'submit',);
  return $type;
}

function theme_imagebutton($element) {
  return '<input type="image" class="form-'. $element['#button_type'] .'" name="'. $element['#name'] .'" value="'. check_plain($element['#default_value']) .'" '. drupal_attributes($element['#attributes']) . ' src="' . $element['#image'] . '" alt="' . $element['#title'] . '" title="' . $element['#title'] . "\" />\n";
}

function imagebutton_value() {
  // null function guarantees default_value doesn't get moved to #value.
}
